
import javax.swing.*;
import javax.swing.border.*;

import java.awt.*;
import java.awt.event.*;

public class VierGewinnt extends JFrame implements ActionListener{
	
	// implemented the game "Vier gewinnt" with test-methods
	
	private final String ACTION = "creategame";  // need for ActionListener
	
	private JPanel panelMaingame = null; // for createGame
	private int numberCols=-1;
	private int numberRows=-1;
	
	private JPanel panelBn=null;
	private JLabel label1;
	private JLabel label2;
	private JSpinner spNumberColumns=null; // get the numbers
	private JSpinner spNumberRows = null;
	JButtonList[] buttonList = null;  // for check
	
	Color colorPlayer = Color.WHITE; // 1-player: red   2-player: blue
	
	public VierGewinnt() {
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setTitle("Vier gewinnt");
		setSize(770,600);
		setLocation(10,10);
		setGUI();
		setVisible(true);
	}
	
	private void setGUI() {
		setLayout( new BorderLayout());
		panelBn = new JPanel(); // panel for the buttons 
		panelBn.setLayout( new FlowLayout());
		add(panelBn,BorderLayout.NORTH);
		
		label1 = new JLabel("Columns");		
		panelBn.add(label1);
		SpinnerNumberModel model1 = new SpinnerNumberModel(5, 4, 20, 1); // start, min, max, increment
		spNumberColumns = new JSpinner(model1);
		panelBn.add(spNumberColumns);
		label2 = new JLabel("Rows");
		
		panelBn.add(label2);		
		SpinnerNumberModel model2 = new SpinnerNumberModel(5, 4, 20, 1);
		spNumberRows = new JSpinner(model2);
		panelBn.add(spNumberRows);
		
		JPanel pn = new JPanel();
		pn.setPreferredSize(new Dimension(30,10));
		panelBn.add(pn);		
		
		addButton(panelBn, ACTION, ACTION);

		// basic JPanel, everytime we remove all cells
		panelMaingame = new JPanel();
		panelMaingame.setBackground(Color.gray);
		panelMaingame.setLayout(new BorderLayout());
		add(panelMaingame,BorderLayout.CENTER);
		setFonts(panelBn,16);
	}  // setGUI
	
	private void addButton(JPanel panelBn, String caption, String command){
		JButton bn = new JButton(caption);
		panelBn.add(bn);
		bn.setActionCommand(command);
		bn.addActionListener(this);		
	}

	private void setFonts(Container cont, int size) {
		for (int i = 0; i < cont.getComponentCount(); i++) {
			Component c = cont.getComponent(i);
			if (c instanceof JPanel)
				setFonts((JPanel) c, size);
			else
				c.setFont(new Font("Arial", Font.BOLD, size));
		}  // for
	}  // setFonts

	

	@Override
	public void actionPerformed(ActionEvent e) {
		String cmd = e.getActionCommand();
		switch (cmd) {
			case ACTION: createGame();
						 break;
			default: if (e.getSource() instanceof JButtonList) {
						JButtonList bn = (JButtonList)e.getSource();
						bn.click(colorPlayer);
						if (colorPlayer == Color.RED)
							colorPlayer = Color.BLUE;
						else
							colorPlayer = Color.RED;
						this.paintAll(this.getGraphics());
						checkGame();
						setButtonBarBackgroundColor(colorPlayer);							
					}
					else {
						Basis.ErrorMsgBox("Wrong case in ActionListener", "Error");
					}
					break;		
			
		} // switch
	} // actionPerformed
	
	private void setButtonBarBackgroundColor(Color color) {
		panelBn.setBackground(color);
		Color c = ( new Color(color.getRGB() ^ 0xFFFFFF ) );
		label1.setForeground(c);
		label2.setForeground(c);
	}
	
	
	private void clearGames() {
		createGame();  // we can also clear all cells
	}
	

	// a) get the numbers of rows and columns
	// b) create the start-button for each columns
	// c) create the cells
	// d) link the buttons to the first row of cells
	// e) link the buttons (links to top, to left, to right, etc.)
	private void createGame() {
		colorPlayer = Color.RED;		
		setButtonBarBackgroundColor(colorPlayer);
		numberCols = (int) spNumberColumns.getValue();
		numberRows = (int) spNumberRows.getValue();
		JPanel panelgame = new JPanel();
		panelgame.setLayout( new GridLayout(numberRows+1, numberCols));
				
		buttonList = new JButtonList[numberCols];  
		for (int col=0; col<numberCols; col++) {
			buttonList[col] = new JButtonList(Integer.toString(col+1),col);  // col, rows
			buttonList[col].setFont(new Font("Arial", Font.BOLD, 20));
			panelgame.add(buttonList[col]);
			buttonList[col].setActionCommand("bn"+col);
			buttonList[col].addActionListener(this);
		}
		
		Cell[] cellsPrevious = new Cell[numberCols]; // cells of the last loop
		Cell[] cells = new Cell[numberCols]; // array for the actual row
		for (int row=0; row<numberRows; row++) {	
			
			for (int col=0; col<numberCols; col++) {
				Cell cell = new Cell(row,col);
				panelgame.add(cell);  // gridlayout, easy to add
				if (row==0) {
					buttonList[col].cellbottom = cell; // link button to cell
				}
				else {
					cellsPrevious[col].cellbottom=cell;
					if (col>0) cellsPrevious[col-1].cellbottomright = cell;
					if (col<(numberCols-1)) cellsPrevious[col+1].cellbottomleft = cell;
				}
				if (col>0) {
					cells[col-1].cellright=cell;  // horizontal check
				}
				cells[col]=cell;
			}  // for col
			cellsPrevious=cells;  // changing the cell-arrays
			cells = new Cell[numberCols];
		}  // for row
		
		panelMaingame.removeAll();
		panelMaingame.add(panelgame,BorderLayout.CENTER);		
		this.paintAll(this.getGraphics());  // updateUI is not working
	}  // createGame
	
	private boolean checkCount(String caption, NumbersPlayerChips numbers) {
		if (numbers.getRed()>=4) {
			disableButtons();
			Basis.Messagebox("Gewonnen hat der rote Spieler", caption);
			return true;
		}
		else if (numbers.getBlue()>=4) {
			disableButtons();
			Basis.Messagebox("Gewonnen hat der blaue Spieler", caption);
			return true;
		}
		else return false;		
	}
	
	private void disableButtons() {
		for (int col=0; col<numberCols; col++) {
			buttonList[col].setEnabled(false);
		}
	}
	
	private void checkGame() {
		NumbersPlayerChips numbers;
		// Check vertical, search for four blue chips or four red chips
		for(JButtonList bn : buttonList) {
			numbers = bn.checkVertical();
			if (checkCount("Gewonnen Spalte",numbers)) return;
		}
		
		
		// Check horizontal,  
		JButtonList bnHorizontal = buttonList[0];
		numbers = bnHorizontal.checkHorizontal();
		if (checkCount("Gewonnen Zeile",numbers)) {
			numbers = bnHorizontal.checkHorizontal();
			return;
		}

		// Check diagonal  top/left  ->  bottom/right, check EVERY diagonale
		JButtonList bnDiagonal1 = buttonList[0];
		numbers = bnDiagonal1.checkDiagonale1();
		if (checkCount("Gewonnen 1. Diagonale",numbers)) {
			numbers = bnDiagonal1.checkDiagonale1();
			return;
		}

		
		
		// Check diagonal  top/right  ->  bottom/left, check EVERY diagonal
		JButtonList bnDiagonal2 = buttonList[ buttonList.length-1 ];
		numbers = bnDiagonal2.checkDiagonale2();
		if (checkCount("Gewonnen 2. Diagonale",numbers)) {
			numbers = bnDiagonal2.checkDiagonale2();
			return;
		}
		
	}  // checkGame	
	
	
	public static void main(String[] args) {
		new VierGewinnt();
	}
	

}
